/*----------------------------------------------------------*
    rslib.h
    RSシリーズプロトコル用API V1.1
                                       Last Edit '13 09/12
   Copyright (c) 2005, 2013 BestTechnology CO.,LTD.
 *----------------------------------------------------------*/

#ifndef _RSLIB_H_INCLUDE
#define _RSLIB_H_INCLUDE

#ifdef __cplusplus
extern "C" {
#endif
#ifdef _WIN32
#include <windows.h>
#else
#include <pthread.h>
#endif

#ifndef _MSC_VER
#include <stdbool.h>
#endif
#include <stdint.h>

#define BROADCASTING_ID     (0xff)

#define RSMAXLENGTH         (180)

#ifndef _WIN32
  typedef int                             HANDLE;
  typedef pthread_mutex_t                 CRITICAL_SECTION;

  #define EnterCriticalSection(mtx)       pthread_mutex_lock(mtx)
  #define LeaveCriticalSection(mtx)       pthread_mutex_unlock(mtx)
  #define DeleteCriticalSection(mtx)      pthread_mutex_destroy(mtx)
  #define InitializeCriticalSection(mtx)  pthread_mutex_init(mtx,NULL)
  #define SetEvent(evt)                   pthread_cond_signal(evt)
  #define INVALID_int_VALUE               -1
  #define INVALID_HANDLE_VALUE            -1
#endif

// デバイスID
#ifdef _WIN32
  #ifdef _WIN64
  typedef uint64_t TDeviceID;
  #else
  typedef uint32_t TDeviceID;
  #endif
#else
  typedef uint32_t TDeviceID;
#endif

// エラーフラグ
typedef uint16_t TErrorCode;
#define ERR_INVALID_DEVID   (1 << 15)
#define ERR_INVALID_ID      (1 << 14)
#define ERR_DIFF_ID         (1 << 13)
#define ERR_ILLEGAL_SIZE    (1 << 12)
#define ERR_INVALID_PARAM   (1 << 11)
#define ERR_COMM            (1 << 10)
#define ERR_CHECKSUM        (1 << 9)
#define ERR_TIMEOUT         (1 << 8)

#define ERR_RS_TEMP         (1 << 7)
#define ERR_RS_TEMPALARM    (1 << 5)
#define ERR_RS_FLASH        (1 << 3)
#define ERR_RS_PACKET       (1 << 1)
#define ERR_NON             (0)

// アラームステータス
typedef struct {
  uint8_t       id;
  TErrorCode  Status;
} TRsAlarmStatus;

#ifdef _WIN32
  #ifdef __MAKE_LIB__
    #define RSAPIDLL __declspec(dllexport) WINAPI
  #else
    #define RSAPIDLL __declspec(dllimport) WINAPI
  #endif
#else
  #define RSAPIDLL
  #define WINAPI
#endif

#ifndef _DYNAMICLOAD
// 静的なDLL呼び出し時は以下のプロトタイプ宣言を使用
TDeviceID RSAPIDLL RS_OpenPort(char *pcom, uint32_t baud);
bool RSAPIDLL RS_ClosePort(TDeviceID dvid);
bool RSAPIDLL RS_SetBaudrate (TDeviceID dvid, uint32_t baud);
bool RSAPIDLL RS_Active(TDeviceID dvid);
void RSAPIDLL RS_SetTimeOutOffset (TDeviceID dvid, uint32_t t);
double RSAPIDLL GetQueryPerformanceCounter (void);
bool RSAPIDLL RS_TxPacket (TDeviceID dvid, uint8_t id, uint8_t cflag, uint8_t *param, uint32_t len, TErrorCode *err);
bool RSAPIDLL RS_RxPacket (TDeviceID dvid, uint8_t *rdata, uint32_t rdatasize, uint32_t *rlen, uint32_t timeout, TErrorCode *err);
bool RSAPIDLL RS_ReadByteData(TDeviceID dvid, uint8_t id, uint8_t adr, uint8_t *rdata, TErrorCode *err);
bool RSAPIDLL RS_WriteByteData(TDeviceID dvid, uint8_t id, uint8_t adr, uint8_t dat, TErrorCode *err);
bool RSAPIDLL RS_ReadWordData(TDeviceID dvid, uint8_t id, uint8_t adr, uint16_t *rdata, TErrorCode *err);
bool RSAPIDLL RS_WriteWordData(TDeviceID dvid, uint8_t id, uint8_t adr, uint16_t dat, TErrorCode *err);
bool RSAPIDLL RS_ReadBlockData (TDeviceID dvid, uint8_t id, uint8_t adr, uint8_t *rdata, uint32_t len, TErrorCode *err);
bool RSAPIDLL RS_WriteBlockData (TDeviceID dvid, uint8_t id, uint8_t adr, uint8_t *dat, uint32_t len, TErrorCode *err);
bool RSAPIDLL RS_Ping(TDeviceID dvid, uint8_t id, TErrorCode *err);
bool RSAPIDLL RS_WriteSyncData (TDeviceID dvid, uint8_t *dat, uint32_t len, TErrorCode *err);
#else
// 動的なDLL呼び出し時は以下の型宣言を使用
typedef TDeviceID (WINAPI *_RS_OpenPort)(char *pcom, uint32_t baud);
typedef bool (WINAPI *_RS_ClosePort)(TDeviceID dvid);
typedef bool (WINAPI *_RS_SetBaudrate) (TDeviceID dvid, uint32_t baud);
typedef bool (WINAPI *_RS_Active)(TDeviceID dvid);
typedef void (WINAPI *_RS_SetTimeOutOffset)(TDeviceID dvid, uint32_t offsettime);
typedef bool (WINAPI *_RS_TxPacket) (TDeviceID dvid, uint8_t id, uint8_t cflag, uint8_t *param, uint32_t len, TErrorCode *err);
typedef bool (WINAPI *_RS_RxPacket) (TDeviceID dvid, uint8_t *rdata, uint32_t rdatasize, uint32_t *rlen, uint32_t timeout, TErrorCode *err);
typedef bool (WINAPI *_RS_ReadByteData)(TDeviceID dvid, uint8_t id, uint8_t adr, uint8_t *rdata, TErrorCode *err);
typedef bool (WINAPI *_RS_WriteByteData)(TDeviceID dvid, uint8_t id, uint8_t adr, uint8_t dat, TErrorCode *err);
typedef bool (WINAPI *_RS_ReadWordData)(TDeviceID dvid, uint8_t id, uint8_t adr, uint16_t *rdata, TErrorCode *err);
typedef bool (WINAPI *_RS_WriteWordData)(TDeviceID dvid, uint8_t id, uint8_t adr, uint16_t dat, TErrorCode *err);
typedef bool (WINAPI *_RS_ReadBlockData)(TDeviceID dvid, uint8_t id, uint8_t adr, uint8_t *rdata, uint32_t len, TErrorCode *err);
typedef bool (WINAPI *_RS_WriteBlockData)(TDeviceID dvid, uint8_t id, uint8_t adr, uint8_t *dat, uint32_t len, TErrorCode *err);
typedef bool (WINAPI *_RS_Ping)(TDeviceID dvid, uint8_t id, TErrorCode *err);
typedef bool (WINAPI *_RS_WriteSyncData) (TDeviceID dvid, uint8_t *dat, uint32_t len, TErrorCode *err);

_RS_OpenPort          RS_OpenPort = NULL;
_RS_ClosePort         RS_ClosePort = NULL;
_RS_SetBaudrate       RS_SetBaudrate = NULL;
_RS_Active            RS_Active = NULL;
_RS_SetTimeOutOffset  RS_SetTimeOutOffset = NULL;
_RS_TxPacket          RS_TxPacket = NULL;
_RS_RxPacket          RS_RxPacket = NULL;
_RS_ReadByteData      RS_ReadByteData = NULL;
_RS_WriteByteData     RS_WriteByteData = NULL;
_RS_ReadWordData      RS_ReadWordData = NULL;
_RS_WriteWordData     RS_WriteWordData = NULL;
_RS_ReadBlockData     RS_ReadBlockData = NULL;
_RS_WriteBlockData    RS_WriteBlockData = NULL;
_RS_WriteSyncData     RS_WriteSyncData = NULL;
_RS_Ping              RS_Ping = NULL;

static HMODULE hModule = NULL;

#ifdef _WIN32
#ifdef _WIN64
#define DLLNAME   "rslib_x64.dll"
#else
#define DLLNAME   "rslib_x32.dll"
#endif
#else
#ifdef __x86_64__
#define DLLNAME   "rslib_x64.dll"
#else
#define DLLNAME   "rslib_x32.dll"
#endif
#endif

// DLLのロード
bool LoadDLL (void) {
  hModule = LoadLibrary (DLLNAME);
  if (hModule == NULL) {
    return false;
  }

  RS_OpenPort         = (_RS_OpenPort)GetProcAddress (hModule, "RS_OpenPort");
  RS_ClosePort        = (_RS_ClosePort)GetProcAddress (hModule, "RS_ClosePort");
  RS_SetBaudrate      = (_RS_SetBaudrate)GetProcAddress (hModule, "RS_SetBaudrate");
  RS_Active           = (_RS_Active)GetProcAddress (hModule, "RS_Active");
  RS_SetTimeOutOffset = (_RS_SetTimeOutOffset)GetProcAddress (hModule, "RS_SetTimeOutOffset");
  RS_TxPacket         = (_RS_TxPacket)GetProcAddress (hModule, "RS_TxPacket");
  RS_RxPacket         = (_RS_RxPacket)GetProcAddress (hModule, "RS_RxPacket");
  RS_ReadByteData     = (_RS_ReadByteData)GetProcAddress (hModule, "RS_ReadByteData");
  RS_WriteByteData    = (_RS_WriteByteData)GetProcAddress (hModule, "RS_WriteByteData");
  RS_ReadWordData     = (_RS_ReadWordData)GetProcAddress (hModule, "RS_ReadWordData");
  RS_WriteWordData    = (_RS_WriteWordData)GetProcAddress (hModule, "RS_WriteWordData");
  RS_ReadBlockData    = (_RS_ReadBlockData)GetProcAddress (hModule, "RS_ReadBlockData");
  RS_WriteBlockData   = (_RS_WriteBlockData)GetProcAddress (hModule, "RS_WriteBlockData");
  RS_WriteSyncData    = (_RS_WriteSyncData)GetProcAddress (hModule, "RS_WriteSyncData");
  RS_Ping             = (_RS_Ping)GetProcAddress (hModule, "RS_Ping");

  return
    (RS_OpenPort         != NULL) &&
    (RS_ClosePort        != NULL) &&
    (RS_SetBaudrate      != NULL) &&
    (RS_Active           != NULL) &&
    (RS_SetTimeOutOffset != NULL) &&
    (RS_TxPacket         != NULL) &&
    (RS_RxPacket         != NULL) &&
    (RS_ReadByteData     != NULL) &&
    (RS_WriteByteData    != NULL) &&
    (RS_ReadWordData     != NULL) &&
    (RS_WriteWordData    != NULL) &&
    (RS_ReadBlockData    != NULL) &&
    (RS_WriteBlockData   != NULL) &&
    (RS_WriteSyncData    != NULL) &&
    (RS_Ping             != NULL);
}

// DLLのアンロード
void UnloadDLL (void) {
  if (hModule != NULL) {
    FreeLibrary (hModule);
    RS_OpenPort         = NULL;
    RS_ClosePort        = NULL;
    RS_SetBaudrate      = NULL;
    RS_Active           = NULL;
    RS_SetTimeOutOffset = NULL;
    RS_TxPacket         = NULL;
    RS_RxPacket         = NULL;
    RS_ReadByteData     = NULL;
    RS_WriteByteData    = NULL;
    RS_ReadWordData     = NULL;
    RS_WriteWordData    = NULL;
    RS_ReadBlockData    = NULL;
    RS_WriteBlockData   = NULL;
    RS_WriteSyncData    = NULL;
    RS_Ping             = NULL;
  }
}

#endif

#ifdef __cplusplus
}
#endif

#endif //_RSLIB_H_INCLUDE
